from Globals import package_home
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName

import os
from os.path import isdir, join
from cStringIO import StringIO
import string

# CHANGE this to use the name of your product - this will be used to ensure
#  we register a skin directory in the correct context
from Products.RubyRed import GLOBALS

# CHANGE this one too - It imports the needed configuration
from Products.RubyRed.config import SKINNAME, BASESKIN, STYLESHEETS, \
                                          JAVASCRIPTS, SELECTSKIN


def setupSkin(self, out, skin_name, base_skin, select_skin,
                         skins_dir='skins', globals=GLOBALS):
    skins_tool = getToolByName(self, 'portal_skins')

    # Add directory views
    addDirectoryViews (skins_tool, 'skins', globals)
    print >> out, "Added directory views to portal_skins."

    # Only add the skin selection if it doesn't already exist
    if skin_name not in skins_tool.getSkinSelections():

        # Get the skin layers of the base skin and convert to an array
        layers = skins_tool.getSkinPath(base_skin)
        layers = map(string.strip, string.split(layers, ','))

        # Get the content of the skins folder
        skins_path = os.path.join(package_home(globals), skins_dir)
        skins_content = os.listdir(skins_path)

        # Add these skin folders to the layers, under 'custom'
        for filename in skins_content:
            # Omit hidden or special filenames
            if filename.startswith('.') or filename in ('CVS', '{arch}'):
                continue
            if isdir(join(skins_path, filename)) and filename not in layers:
                try:
                    layers.insert(layers.index('custom')+1, filename)
                except ValueError:
                    layers.insert(0, filename)

        # Add our skin selection
        layers = ', '.join(layers)
        skins_tool.addSkinSelection(skin_name, layers)
        print >> out, "Added skin selection to portal_skins."
        
        # Activate the skin selection
        if select_skin:
            skins_tool.manage_properties(default_skin=skin_name)
    else:
        print >> out, "Skin selection already exists, leaving it alone."

def registerStylesheets(self, out, stylesheets):
    # register additional CSS stylesheets with portal_css
    csstool = getToolByName(self, 'portal_css')
    # I coudn't find any method to list registered stylesheets so I don't
    # test if they are already registered before trying to register them
    for ss in stylesheets:
        csstool.registerStylesheet(**ss)
    print >> out, "installed the Plone additional stylesheets."

def registerScripts(self, out, scripts):
    # register additional JS Scripts with portal_javascripts
    jstool = getToolByName(self, 'portal_javascripts')
    for js in scripts:
        jstool.registerScript(**js)
    print >> out, "installed the Plone additional javascripts."

def install(self):
    out = StringIO()

    setupSkin(self, out, SKINNAME, BASESKIN, SELECTSKIN)
    registerStylesheets(self, out, STYLESHEETS)
    registerScripts(self, out, JAVASCRIPTS)
    
    print >> out, "Installation completed."
    return out.getvalue()


def removeSkin(self, out, skin_name, base_skin):
    # Delete the portal_skins property which holds the skin selection
    skins_tool = getToolByName(self, 'portal_skins')
    if skin_name in skins_tool.getSkinSelections():
        # Remove skin selection from portal_skins
        skins_tool.manage_skinLayers(del_skin=1, chosen=(skin_name,))
        # select the default skin selection
        skins_tool.manage_properties(default_skin=base_skin)
        print >> out, "Removed skin selection from portal skins."

def unregisterResources(self, out, resourceIds, toolname):
    # Unregister stylesheets and javascripts from respective tools
    tool = getToolByName(self, toolname)
    for resId in resourceIds:
        tool.unregisterResource(resId)
    print >> out, "Unregistered resources from %s." % toolname

def uninstall(self):
    out = StringIO()

    removeSkin(self, out, SKINNAME, BASESKIN)
    stylesheetnames = [ ss['id'] for ss in STYLESHEETS ]
    unregisterResources(self, out, stylesheetnames, 'portal_css')
    javascriptnames = [ js['id'] for js in JAVASCRIPTS ]
    unregisterResources(self, out, javascriptnames, 'portal_javascripts')
